#pragma once

#include <string_view>

inline constexpr std::string_view EXAMPLE_CONFIG = R"#(

# This config is a STUB! This should never be generated.
# Use the default lua config from https://github.com/hyprwm/Hyprland/blob/main/example/hyprland.lua

autogenerated = 1

$mainMod = SUPER
$terminal = kitty
$fileManager = dolphin
$menu = hyprlauncher

bind = $mainMod, Q, exec, $terminal
bind = $mainMod, C, killactive,
bind = $mainMod, M, exec, command -v hyprshutdown >/dev/null 2>&1 && hyprshutdown || hyprctl dispatch exit
bind = $mainMod, E, exec, $fileManager
bind = $mainMod, V, togglefloating,
bind = $mainMod, R, exec, $menu
)#";
