#pragma once

#include <hyprgraphics/resource/AsyncResourceGatherer.hpp>
#include <hyprgraphics/resource/resources/ImageResource.hpp>

#include "../helpers/memory/Memory.hpp"

inline UP<Hyprgraphics::CAsyncResourceGatherer> g_pAsyncResourceGatherer;