#pragma once
#define GIT_COMMIT_HASH    "39d7e209c79d451efab1b21151d5938289da838d"
#define GIT_BRANCH         "v0.55.2"
#define GIT_COMMIT_MESSAGE "version: bump to 0.55.2"
#define GIT_COMMIT_DATE    "Sat May 16 12:59:19 2026"
#define GIT_DIRTY          "clean"
#define GIT_TAG            "v0.55.2"
#define GIT_COMMITS        "7319"

#define AQUAMARINE_VERSION "0.11.0"
// clang-format off
#define AQUAMARINE_VERSION_MAJOR 0
#define AQUAMARINE_VERSION_MINOR 11
#define AQUAMARINE_VERSION_PATCH 0
// clang-format on
#define HYPRLANG_VERSION     "0.6.8"
#define HYPRUTILS_VERSION    "0.13.1"
#define HYPRCURSOR_VERSION   "0.1.13"
#define HYPRGRAPHICS_VERSION "0.5.1"
